package org.jeecg.modules.ctop.service.impl;

import lombok.extern.slf4j.Slf4j;
import org.jeecg.modules.ctop.service.IFfmpegService;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
@Slf4j
public class FfmpegServiceImpl implements IFfmpegService {
    /**
     * FFmpeg全路径
     */
    @Value("${jeecg.ffmpeg.bin}")
    private String FFMPEG_PATH;

    /**
     * 从视频中抽取音频文件
     */
    public void getM4aFromMp4(String videoPath, String m4apath) {
        /**
         * D:\java\ffmpeg\bin\ffmpeg.exe -i D:\data\test\\video.mp4 -vn -y -acodec copy D:\data\test\\video.m4a
         */
        List<String> command = new ArrayList<>();
        command.add(FFMPEG_PATH);
        command.add("-i");
        command.add(videoPath);
        command.add("-vn");
        command.add("-y");
        command.add("-acodec");
        command.add("copy");
        command.add(m4apath);
        runCommand(command);
    }

    /**
     * 截取音频文件
     */
    public void cutM4a(String srcPath, String targetPath, String startSecond, String endSecond) {
        /**
         *D:\java\ffmpeg\bin\ffmpeg.exe -i D:\data\test\\video.m4a -vn -acodec copy -ss 00:00:00.000 -t 00:00:08.000 D:\data\test\\cut.m4a
         */
        List<String> command = new ArrayList<>();
        command.add(FFMPEG_PATH);
        command.add("-i");
        command.add(srcPath);
        command.add("-vn");
        command.add("-acodec");
        command.add("copy");
        command.add("-ss");
        command.add(startSecond);
        command.add("-t");
        command.add(endSecond);
        command.add(targetPath);
        runCommand(command);
    }

    /**
     * 音频拼接
     * D:\java\ffmpeg\bin\ffmpeg.exe -i D:\data\test\\cut.m4a -i D:\data\test\\fill.m4a -filter_complex "[0:0] [1:0] concat=n=2:v=0:a=1 [a]" -map [a] D:\data\test\\final.m4a
     */
    public void audioMosaic(String audioPart1, String audioPart2, String finalAudio) {
        List<String> command = new ArrayList<>();
        command.add(FFMPEG_PATH);
        command.add("-i");
        command.add(audioPart1);
        command.add("-i");
        command.add(audioPart2);
        command.add("-filter_complex");
        command.add("\"[0:0] [1:0] concat=n=2:v=0:a=1 [a]\"");
        command.add("-map");
        command.add("[a]");
        command.add(finalAudio);
        runCommand(command);
    }

    /**
     * 音频+视频合成视频
     *
     * @param srcVideoPath
     * @param srcAudioPath
     * @param finalVideoPath D:\java\ffmpeg\bin\ffmpeg.exe -i D:\data\test\\merge.mp4 -i D:\data\test\\final.m4a -c:v copy -c:a aac -strict experimental D:\data\test\\final.mp4
     */
    public void mergeAudioAndVideo(String srcVideoPath, String srcAudioPath, String finalVideoPath) {
        List<String> command = new ArrayList<>();
        command.add(FFMPEG_PATH);
        command.add("-i");
        command.add(srcVideoPath);
        command.add("-i");
        command.add(srcAudioPath);
        command.add("-c:v");
        command.add("copy");
        command.add("-c:a");
        command.add("aac");
        command.add("-strict");
        command.add("experimental");
        command.add(finalVideoPath);
        runCommand(command);
    }

    /**
     * D:\java\ffmpeg\bin\ffmpeg.exe -f image2 -i D:\\data\\video\\%d.jpg  -vcodec libx264 -r 25  D:\\data\\merge.mp4
     *
     * @param imagePath
     * @param frameNumber
     * @param videoFilePath
     */
    public void MergeImageToVideo(String imagePath, Integer frameNumber, String videoFilePath) {
        List<String> command = new ArrayList<>();
        command.add(FFMPEG_PATH);
        command.add("-i");
        command.add("image2");
        command.add("-i");
        command.add(imagePath + "%d.jpg");
        command.add("-vcodec");
        command.add("copy");
        command.add("libx264");
        command.add("-r");
        command.add(frameNumber + "");
        command.add(videoFilePath);
        runCommand(command);
    }

    private void runCommand(List<String> command) {
        try {
            ProcessBuilder builder = new ProcessBuilder();
            builder.command(command);
            builder.redirectErrorStream(true);
            Process process = builder.start();
            // 等待进程执行结束
            process.waitFor();
            log.info("ffmpeg命令执行完成");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
